-- Provides additional features to the map type for this quest.

require("scripts/multi_events")

local map_meta = sol.main.get_metatable("map")

map_meta:register_event("on_started", function(map)
  for entity in map:get_entities() do
    for _, prop in ipairs(entity:get_properties()) do
      if prop.key == "remove_on_start_variable" then
        if map:get_game():get_value(prop.value) == true then
          entity:remove()
        end
      end
    end
  end
end)

function map_meta:move_camera(x, y, speed, callback, delay_before, delay_after)

  local camera = self:get_camera()
  local game = self:get_game()
  local hero = self:get_hero()

  delay_before = delay_before or 1000
  delay_after = delay_after or 1000

  local back_x, back_y = camera:get_position_to_track(hero)
  game:set_suspended(true)
  camera:start_manual()

  local movement = sol.movement.create("target")
  movement:set_target(camera:get_position_to_track(x, y))
  movement:set_ignore_obstacles(true)
  movement:set_speed(speed)
  movement:start(camera, function()
    local timer_1 = sol.timer.start(self, delay_before, function()
      if callback ~= nil then
        callback()
      end
      local timer_2 = sol.timer.start(self, delay_after, function()
        local movement = sol.movement.create("target")
        movement:set_target(back_x, back_y)
        movement:set_ignore_obstacles(true)
        movement:set_speed(speed)
        movement:start(camera, function()
          game:set_suspended(false)
          camera:start_tracking(hero)
          if self.on_camera_back ~= nil then
            self:on_camera_back()
          end
        end)
      end)
      timer_2:set_suspended_with_map(false)
    end)
    timer_1:set_suspended_with_map(false)
  end)
end

return true
